import javax.swing.JPanel;
import data.DataRepresentation;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.geom.*;
import java.awt.event.*;

import drawing.*;


public class Legend extends Panel {



  //Konstanten
  private static final int ROWHEIGHT = 30;
  private static final int BORDER = 20;
  private static final int MARKWIDTH = 40;
  private static final int MARKHEIGHT = 20;



  //Zeichnen
  private boolean firsttime;
  private BufferedImage bi;
  private Graphics2D big;
  private Rectangle area;

  //Legendengre
  private int width, height, minheight;



  //Daten
  private String[] colnames;
  private ColorList cl;



  public Legend(int width, int minheight, DataRepresentation datrep, ColorList cl) {
    this.width = width;
    this.minheight = minheight;
    this.cl = cl;

    setDataRepresentation(datrep);

  }

  //setzt neuen Datensatz
  public void setDataRepresentation(DataRepresentation datrep) {
    if(datrep != null) {
      colnames = datrep.getColumnNames();
    }
    else {
      colnames = null;
    }


    //Hhenfestlegung
    if (colnames != null) {
      this.height = ROWHEIGHT*colnames.length + 2*BORDER;
    }
    else {
      this.height = 2*BORDER;
    }
    if (height < minheight) this.height = minheight;

  }

  public void setColorList(ColorList cl) {
    this.cl = cl;
  }

  public ColorList getColorList() {
    return cl;
  }


  public BufferedImage getImage() {
    BufferedImage bimg = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
    Graphics2D big = bimg.createGraphics();
    big.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    big.setFont(new Font("Arial", Font.PLAIN, 12));


    // Clears the rectangle that was previously drawn.
    big.setColor(Color.WHITE);
    big.fillRect(0, 0, width, height);


    if (colnames != null) {
      for(int i = 0; i < colnames.length; i++) {
        int ypos = BORDER + i*ROWHEIGHT + (ROWHEIGHT-MARKHEIGHT)/2;

        //Farbkasten
        big.setColor(cl.getColor(i));
        big.fill(new Rectangle(BORDER, ypos, MARKWIDTH, MARKHEIGHT));


        //Beschriftung
        FontMetrics metrics = big.getFontMetrics();
        int fontheight = metrics.getHeight();

        String currstring = colnames[i];

        while(metrics.stringWidth(currstring) > (width - (2.5*BORDER + MARKWIDTH))) {
          currstring = currstring.substring(0, currstring.length()-3) + ".";
        }


        big.setColor(Color.BLACK);
        big.drawString(currstring, (float)(1.5*BORDER + MARKWIDTH),
        (float)(ypos + fontheight + ((MARKHEIGHT-fontheight)/2) - 2));

      }
    }




    return bimg;
  }



  //fgt Legende am unteren Bildrand hinzu.
  public BufferedImage embedLegend(BufferedImage bimg) {
    if (colnames == null) return bimg;

    //Bildgre
    int bimgwidth = bimg.getWidth();
    int bimgheight = bimg.getHeight();


    //Berechnung wieviele Zeilen fr die Legende bentigt werden
    int fulllength = width*colnames.length;
    int rownumber = (int)Math.ceil((double)fulllength/(double)bimgwidth);


    //Erstellen eines neuen Bildes und dessen Graphics-Objekts
    BufferedImage newimg = new BufferedImage(bimgwidth, bimgheight + rownumber*ROWHEIGHT + 20,
    BufferedImage.TYPE_INT_RGB);

    Graphics2D g2d = newimg.createGraphics();
    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g2d.setFont(new Font("Arial", Font.PLAIN, 12));

    //Zeichnen des alten Bildes und einer Trennungslinie
    g2d.setColor(Color.WHITE);
    g2d.fillRect(0,0, newimg.getWidth(), newimg.getHeight());

    g2d.drawImage(bimg, 0, 0, this);

    g2d.setColor(Color.BLACK);
    g2d.fillRect(0,bimgheight, newimg.getWidth(), 5);



    //Zeichnen der Legende
    int startxpos = 0;
    int startypos = bimgheight + 15;

    for(int i = 0; i < colnames.length; i++) {

      System.out.println("startx: " + startxpos + ", starty: " + startypos);

      //Farbkasten
      g2d.setColor(cl.getColor(i));
      g2d.fill(new Rectangle(startxpos + BORDER, startypos, MARKWIDTH, MARKHEIGHT));


      //Beschriftung
      FontMetrics metrics = g2d.getFontMetrics();
      int fontheight = metrics.getHeight();

      String currstring = colnames[i];

      while(metrics.stringWidth(currstring) > (width - (2.5*BORDER + MARKWIDTH))) {
        currstring = currstring.substring(0, currstring.length()-3) + ".";
      }


      g2d.setColor(Color.BLACK);
      g2d.drawString(currstring, (float)(startxpos + 1.5*BORDER + MARKWIDTH),
      (float)(startypos + fontheight + ((MARKHEIGHT-fontheight)/2) - 2));

      startxpos += width;

      if (startxpos + width > newimg.getWidth()) {
        startxpos = 0;
        startypos += ROWHEIGHT;
      }


    }

    return newimg;
  }



  //Retourniert die Hhe, die fr das einbetten der Legende bentigt wird.
  public int getEmbedingHeight(int imagewidth) {

    if (colnames == null) return 0;

    //Berechnung wieviele Zeilen fr die Legende bentigt werden
    int fulllength = width*colnames.length;
    int rownumber = (int)Math.ceil((double)fulllength/(double)imagewidth);

    return (rownumber*ROWHEIGHT + 20);
  }







}