
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;

import java.io.File;
import javax.imageio.ImageIO;


import data.*;
import drawing.*;
import export.*;


//class creates and handles the functionality the interface of the application
public class Mainframe extends JFrame implements WindowListener, IMainframe{

  //Konstanten
  static final int LEGENDWIDTH = 200;
  static final int BORDER = 20;

  //Klassenvariablen
  Container c;
  Plotter plotter;
  JScrollPane legendpane, plotterpane;
  Legend legend;

  //Men
  JMenuBar menubar;

  //Daten
  DataRepresentation datrep = null;



  //MagicLens - MenuItems
  JMenuItem mlenscreate, mlensdelete;
  JMenu mlenssubinter, mlenssubrepr;
  JRadioButtonMenuItem mlenssubreprhisto, mlenssubreprriver;



/**
  * Konstruktor, Erzeugen des GUI
  */
  public Mainframe() {

    //Window-Properties
    setResizable(false);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    addWindowListener(this);


    //create Menu
    menubar = new JMenuBar();
    menubar.add(createDataMenu());
    menubar.add(createDesignMenu());
    menubar.add(createLensMenu());
    setJMenuBar(menubar);

    //Containers-Properties
    c = this.getContentPane();
    this.setSize(new Dimension(800, 600));
    c.setLayout(null);
    this.setTitle("ThemeRiver");

    //Zeichenflche
    ColorList startcl;
    try {
      startcl = (new ColorProperties()).getColorList("Standard");
    }
    catch (Exception e) {
      startcl = new ColorList("Standard");
    }

    plotter = new Plotter(getWidth()-3*BORDER-LEGENDWIDTH,getHeight() - 4*BORDER, startcl);
    plotter.setLocation(BORDER,BORDER);

    plotterpane = new JScrollPane(plotter);
    plotterpane.setLocation(BORDER,BORDER);
    plotterpane.setSize(getWidth()-3*BORDER-LEGENDWIDTH,getHeight() - 4*BORDER);


    //Legende
    legend = new Legend(LEGENDWIDTH, (getHeight()-4*BORDER)/2, null, startcl);

    legendpane = new JScrollPane(new JLabel(new ImageIcon(legend.getImage())));
    legendpane.setPreferredSize(new Dimension(LEGENDWIDTH, (getHeight()-4*BORDER)/2));
    legendpane.setSize(new Dimension(LEGENDWIDTH, (getHeight()-4*BORDER)/2));
    legendpane.setLocation(getWidth()-LEGENDWIDTH-BORDER, BORDER);
    legendpane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);


    //Add Elements to Container
    c.add(plotterpane);
    c.add(legendpane);
  }



  /**
    * Erzeugt das DatenMen des Fensters
    */
  private JMenu createDataMenu() {
    JMenu mdata = new JMenu("Daten");
    //Ansprechen mit Alt+D
    mdata.setMnemonic(KeyEvent.VK_D);

    //Menpunkt Laden
    JMenuItem mdataload = new JMenuItem("Laden");
    mdataload.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        loadData();
      }
    });

    //Ausfhrbar mit Strg+o
    setCtrlAccelerator(mdataload, 'O');


    //Menpunkt Exportieren
    JMenuItem mdataexp = new JMenuItem("Bild exportieren");
    mdataexp.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        exportImage();
      }
    });


    //Menpunkt Exportieren
    JMenuItem mdataexpprop = new JMenuItem("Exporteinstellungen");
    mdataexpprop.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showExportPropDialog();
      }
    });






    mdata.add(mdataload);
    mdata.addSeparator();
    mdata.add(mdataexp);
    mdata.add(mdataexpprop);
    return mdata;
  }



  /**
    * Erzeugt das den Datensatz Laden Menpunkt
    */
  private void loadData() {
    JFileChooser chooser = new JFileChooser(new File("daten_files"));
    ExampleFileFilter filter = new ExampleFileFilter();
    filter.addExtension("txt");
    filter.setDescription("Textdatei");
    chooser.setFileFilter(filter);
    int returnVal = chooser.showOpenDialog(this);
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      DataLoader loader = new DataLoader(chooser.getSelectedFile());
      datrep = loader.getDataRepresentation();
      plotter.setDataRepresentation(datrep);
      legend.setDataRepresentation(datrep);
      legendpane.setViewportView(new JLabel(new ImageIcon(legend.getImage())));
      //legendpane.revalidate();
    }
  }



  /**
    * Exportiert die aktuelle Visualisierung gem den Exporteinstellungen
    */
  private void exportImage() {

    //Erstellen eines neuen Datei-Auswahlfensters
    JFileChooser chooser = new JFileChooser(new File("bild_files"));

    //Festlegen der Dateitypen, die auswhlbar sind.
    ExampleFileFilter filter1 = new ExampleFileFilter();
    filter1.addExtension("png");
    filter1.setDescription("Portierbare Netzwerk-Grafik");
    ExampleFileFilter filter2 = new ExampleFileFilter();
    filter2.addExtension("jpg");
    filter2.setDescription("JPEG Bitmap");

    //Setzen von Parametern fr den FileChooser
    chooser.setAcceptAllFileFilterUsed(false);
    chooser.addChoosableFileFilter(filter2);
    chooser.addChoosableFileFilter(filter1);
    int returnVal = chooser.showSaveDialog(this);


    //Wenn ein Dateiname ausgewhlt wurde.
    if(returnVal == JFileChooser.APPROVE_OPTION) {
      try {

        //Laden der ExportProperties
        ExportProperties expprop = new ExportProperties();


        //Erzeugen eines neuen Plotters in der entsprechenden Gre
        Plotter newplotter;
        if (expprop.getLegendFlag()) {
          int newplotterheight = expprop.getHeight()-legend.getEmbedingHeight(expprop.getWidth());
          if (newplotterheight < 10) newplotterheight = 10;
          newplotter = new Plotter(plotter, expprop.getWidth(), newplotterheight);
        }
        else {
          newplotter = new Plotter(plotter, expprop.getWidth(), expprop.getHeight());
        }


        //Abrufen des Bildes
        BufferedImage bimg = newplotter.getImage();

        if (expprop.getLegendFlag()) {
          bimg = legend.embedLegend(bimg);
        }

        //Festlegen des Filenamens (mit Extensions, etc.)
        File file = chooser.getSelectedFile();
        String extension = file.getName().substring(file.getName().length()-4);

        ExampleFileFilter currfilt = (ExampleFileFilter)chooser.getFileFilter();
        if (currfilt.equals(filter1)) {
          if (!(extension.equals(".png"))) {
            file = new File(file.getAbsolutePath() + ".png");
          }
        }
        else {
          if (!(extension.equals(".jpg"))) {
            file = new File(file.getAbsolutePath() + ".jpg");
          }
        }



        //Exportieren des Bildes.
        ImageIO.write(bimg, filter1.getExtension(file), file);
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }



  }



  /**
    * ffnet das Interface fr die Exporteinstellungen
    */
  private void showExportPropDialog() {
    ExportPropDialog epd = new ExportPropDialog(this);

    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = epd.getSize();
    if (frameSize.height > screenSize.height) {
        frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width) {
        frameSize.width = screenSize.width;
    }
    epd.setLocation((screenSize.width - frameSize.width)/2, (screenSize.height - frameSize.height) / 2);
    epd.setVisible(true);
  }


  /**
    * Erzeugt das Darstellungsart-Men des Fensters
    */
  private JMenu createDesignMenu() {

    //Darstellungsart-Men
    JMenu subrepr = new JMenu("Darstellungsart");

    //Menpunkt ThemeRiver
    JRadioButtonMenuItem subreprriver = new JRadioButtonMenuItem("ThemeRiver");
    subreprriver.setSelected(true);
    subreprriver.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setRepresentationStyle(Plotter.RIVER_REPRESENTATION);
      }
    });

    //Menpunkt Histogramm
    JRadioButtonMenuItem subreprhisto = new JRadioButtonMenuItem("Histogramm");
    subreprhisto.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setRepresentationStyle(Plotter.HISTO_REPRESENTATION);
      }
    });


    ButtonGroup bgroup1 = new ButtonGroup();
    bgroup1.add(subreprriver);
    bgroup1.add(subreprhisto);

    subrepr.add(subreprriver);
    subrepr.add(subreprhisto);


    //Interpolationsart - Men
    JMenu subinter = new JMenu("Interpolationsart");

    //Menpunkt "Linear"
    JRadioButtonMenuItem minterlin = new JRadioButtonMenuItem("Linear");
    minterlin.setSelected(true);
    minterlin.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setInterpolationStyle(Plotter.LINEAR_INTERPOLATION);
      }
    });


    //Menpunkt "Cubic Splines"
    JRadioButtonMenuItem minterspline = new JRadioButtonMenuItem("Cubic Splines");
    minterspline.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setInterpolationStyle(Plotter.NORMAL_SPLINE_INTERPOLATION);
      }
    });


    //Menpunkt "Cubic Splines"
    JRadioButtonMenuItem mintersplinezero = new JRadioButtonMenuItem("Cubic Splines (zero-deviation)");
    mintersplinezero.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setInterpolationStyle(Plotter.ZEROSLOPE_SPLINE_INTERPOLATION);
      }
    });


    //Menpunkt "Cubic Splines"
    JRadioButtonMenuItem mintersplinemean = new JRadioButtonMenuItem("Cubic Splines (mean-deviation)");
    mintersplinemean.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        plotter.setInterpolationStyle(Plotter.MEANSLOPE_SPLINE_INTERPOLATION);
      }
    });

    ButtonGroup bgroup2 = new ButtonGroup();
    bgroup2.add(minterlin);
    bgroup2.add(minterspline);
    bgroup2.add(mintersplinezero);
    bgroup2.add(mintersplinemean);

    subinter.add(minterlin);
    subinter.add(minterspline);
    subinter.add(mintersplinezero);
    subinter.add(mintersplinemean);



    //Datenpunktestil - Men
    JMenu subpoints = new JMenu("Datenpunkte");

    //Menpunkt "Keine Datenpunkte"
    JRadioButtonMenuItem mpointsno = new JRadioButtonMenuItem("Keine");
    mpointsno.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setPointStyle(Plotter.NO_POINTS);
      }
    });


    //Menpunkt "Punkte"
    JRadioButtonMenuItem mpointssimple = new JRadioButtonMenuItem("Punkte");
    mpointssimple.setSelected(true);
    mpointssimple.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setPointStyle(Plotter.SIMPLE_POINTS);
      }
    });


    //Menpunkt "Punkte und Daten"
    JRadioButtonMenuItem mpointsdata = new JRadioButtonMenuItem("Punkte und Daten");
    mpointsdata.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setPointStyle(Plotter.DATA_POINTS);
      }
    });


    ButtonGroup bgroup3 = new ButtonGroup();
    bgroup3.add(mpointsno);
    bgroup3.add(mpointssimple);
    bgroup3.add(mpointsdata);

    subpoints.add(mpointsno);
    subpoints.add(mpointssimple);
    subpoints.add(mpointsdata);




    //Menpunkt "Farbeinstellungen"
    JMenuItem mcolorprop = new JMenuItem("Farbeinstellungen");
    mcolorprop.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        openColorDialog();
      }
    });


    //Men, das retourniert wird
    JMenu repr = new JMenu("Darstellung");

    repr.add(subrepr);
    repr.add(subinter);
    repr.add(subpoints);
    repr.addSeparator();
    repr.add(mcolorprop);

    return repr;
  }


  /**
  * ffnet das Farbeinstellungsfenster
  */
  private void openColorDialog() {
    ColorDialog cd = new ColorDialog(this, legend.getColorList());

    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = cd.getSize();
    if (frameSize.height > screenSize.height) {
        frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width) {
        frameSize.width = screenSize.width;
    }
    cd.setLocation((screenSize.width - frameSize.width)/2, (screenSize.height - frameSize.height) / 2);
    cd.setVisible(true);
  }


  /**
  * Erzeugt das Magic-Lens-Men des Fensters
  */
  private JMenu createLensMenu() {

    //Darstellungsart-Men
    JMenu mlens = new JMenu("Magic Lens");

    //Menpunkt "Erstellen"
    mlenscreate = new JMenuItem("Erstellen");
    mlenscreate.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setMagicLens();
        mlensdelete.setEnabled(true);
        mlenssubinter.setEnabled(true);
        mlenssubrepr.setEnabled(true);
        mlenscreate.setEnabled(false);
      }
    });


    //Menpunkt "Lschen"
    mlensdelete = new JMenuItem("Lschen");
    mlensdelete.setEnabled(false);
    mlensdelete.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.deleteMagicLens();
        mlenscreate.setEnabled(true);
        mlensdelete.setEnabled(false);
        mlenssubinter.setEnabled(false);
        mlenssubrepr.setEnabled(false);
      }
    });


    //Darstellungsart-Men
    mlenssubrepr = new JMenu("Darstellungsart");
    mlenssubrepr.setEnabled(false);

    //Menpunkt ThemeRiver
    mlenssubreprriver = new JRadioButtonMenuItem("ThemeRiver");
    mlenssubreprriver.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setMagicLensRepresentationStyle(Plotter.RIVER_REPRESENTATION);
      }
    });

    //Menpunkt Histogramm
    mlenssubreprhisto = new JRadioButtonMenuItem("Histogramm");
    mlenssubreprhisto.setSelected(true);
    mlenssubreprhisto.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setMagicLensRepresentationStyle(Plotter.HISTO_REPRESENTATION);
      }
    });


    ButtonGroup mlensbgroup1 = new ButtonGroup();
    mlensbgroup1.add(mlenssubreprriver);
    mlensbgroup1.add(mlenssubreprhisto);

    mlenssubrepr.add(mlenssubreprriver);
    mlenssubrepr.add(mlenssubreprhisto);


    //Interpolationsart - Men
    mlenssubinter = new JMenu("Interpolationsart");
    mlenssubinter.setEnabled(false);

    //Menpunkt "Linear"
    JRadioButtonMenuItem mlensminterlin = new JRadioButtonMenuItem("Linear");
    mlensminterlin.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setMagicLensInterpolationStyle(Plotter.LINEAR_INTERPOLATION);
      }
    });


    //Menpunkt "Cubic Splines"
    JRadioButtonMenuItem mlensminterspline = new JRadioButtonMenuItem("Cubic Splines");
    mlensminterspline.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setMagicLensInterpolationStyle(Plotter.NORMAL_SPLINE_INTERPOLATION);
      }
    });


    //Menpunkt "Cubic Splines"
    JRadioButtonMenuItem mlensmintersplinezero = new JRadioButtonMenuItem("Cubic Splines (zero-deviation)");
    mlensmintersplinezero.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        plotter.setMagicLensInterpolationStyle(Plotter.ZEROSLOPE_SPLINE_INTERPOLATION);
      }
    });


    //Menpunkt "Cubic Splines"
    JRadioButtonMenuItem mlensmintersplinemean = new JRadioButtonMenuItem("Cubic Splines (mean-deviation)");
    mlensmintersplinemean.setSelected(true);
    mlensmintersplinemean.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        plotter.setMagicLensInterpolationStyle(Plotter.MEANSLOPE_SPLINE_INTERPOLATION);
      }
    });

    ButtonGroup mlensbgroup2 = new ButtonGroup();
    mlensbgroup2.add(mlensminterlin);
    mlensbgroup2.add(mlensminterspline);
    mlensbgroup2.add(mlensmintersplinezero);
    mlensbgroup2.add(mlensmintersplinemean);

    mlenssubinter.add(mlensminterlin);
    mlenssubinter.add(mlensminterspline);
    mlenssubinter.add(mlensmintersplinezero);
    mlenssubinter.add(mlensmintersplinemean);





    mlens.add(mlenscreate);
    mlens.add(mlensdelete);
    mlens.addSeparator();
    mlens.add(mlenssubrepr);
    mlens.add(mlenssubinter);

    return mlens;
  }






  //Setzt einen Shortcut um MenPunkte auszufhren
  private void setCtrlAccelerator(JMenuItem mi, char acc) {
    KeyStroke ks = KeyStroke.getKeyStroke(acc, Event.CTRL_MASK);
    mi.setAccelerator(ks);
  }


  //WindowListener
  public void windowClosing(WindowEvent event1) {
    System.exit(0);
  }
  public void windowGainedFocus(WindowEvent event2) {}
  public void windowClosed(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}



  //IMainframe
  public void setColorList(ColorList cl) {
    plotter.setColorList(cl);
    legend.setColorList(cl);
    legendpane.setViewportView(new JLabel(new ImageIcon(legend.getImage())));

  }
}